from django_statsd.middleware import start, stop, with_, wrapper, decorator
from django_statsd import redis, celery, json

__all__ = ['start', 'stop', 'with_', 'wrapper', 'decorator', 'json', 'redis',
    'celery']




from __future__ import absolute_import
import django_statsd

try:
    import json

    # NOTE issubclass is true if both are the same class
    if not hasattr(json, 'statsd_patched'):
        json.statsd_patched = True
        json.load = django_statsd.wrapper('json', json.load)
        json.loads = django_statsd.wrapper('json', json.loads)
        json.dump = django_statsd.wrapper('json', json.dump)
        json.dumps = django_statsd.wrapper('json', json.dumps)
except ImportError:
    print 'no json'
    pass

try:
    import cjson

    # NOTE issubclass is true if both are the same class
    if not hasattr(json, 'statsd_patched'):
        cjson.statsd_patched = True
        cjson.encode = django_statsd.wrapper('cjson', cjson.encode)
        cjson.decode = django_statsd.wrapper('cjson', cjson.decode)
except ImportError:
    print 'no cjson'
    pass




from __future__ import absolute_import
from django_statsd import middleware

try:
    from celery.signals import task_prerun, task_postrun, task_failure

    def start(**kwargs):
        middleware.StatsdMiddleware.start('celery')

    def stop(**kwargs):
        middleware.StatsdMiddleware.stop(kwargs.get('task').name)
        middleware.StatsdMiddleware.scope.timings = None

    def clear(**kwargs):
        middleware.StatsdMiddleware.scope.timings = None

    task_prerun.connect(start)
    task_postrun.connect(stop)
    task_failure.connect(clear)

except ImportError:
    pass




import statsd
from django.conf import settings

def get_connection(host=None, port=None, sample_rate=None):
    if not host:
        host = getattr(settings, 'STATSD_HOST', '127.0.0.1')

    if not port:
        port = getattr(settings, 'STATSD_PORT', 8125)

    if not sample_rate:
        sample_rate = getattr(settings, 'STATSD_SAMPLE_RATE', 1.0)

    return statsd.Connection(host, port, sample_rate)

def get_client(name, connection=None, class_=statsd.Client):
    if not connection:
        connection = get_connection()

    return class_(name, connection)

def get_timer(name, connection=None):
    return get_client(name, connection, statsd.Timer)

def get_counter(name, connection=None):
    return get_client(name, connection, statsd.Counter)




from __future__ import with_statement
import time
import logging
import functools
import threading
import warnings
import collections
import statsd

from django.conf import settings

from django_statsd import utils

logger = logging.getLogger(__name__)


class WithTimer(object):

    def __init__(self, timer, key):
        self.timer = timer
        self.key = key

    def __enter__(self):
        self.timer.start(self.key)

    def __exit__(
            self,
            type_,
            value,
            traceback,
        ):
        self.timer.stop(self.key)


class Client(object):
    class_ = statsd.Client

    def __init__(self, prefix='view'):
        self.prefix = prefix
        self.data = collections.defaultdict(int)

    def get_client(self, *args):
        prefix = '%s.%s' % (self.prefix, '.'.join(args))
        return utils.get_client(prefix, class_=self.class_)

    def submit(self, *args):
        raise NotImplementedError(
            'Subclasses must define a `submit` function')

class Counter(Client):
    class_ = statsd.Counter

    def increment(self, key, delta=1):
        self.data[key] += delta

    def decrement(self, key, delta=1):
        self.data[key] -= delta

    def submit(self, *args):
        client = self.get_client(*args)
        for k, v in self.data:
            if v:
                client.increment(k, v)

class Timer(Client):
    class_ = statsd.Timer

    def __init__(self, prefix='view'):
        Client.__init__(self, prefix)
        self.starts = {}
        self.data = collections.defaultdict(float)

    def start(self, key):
        assert key not in self.starts, 'Already started tracking %s' % key
        self.starts[key] = time.time()

    def stop(self, key):
        assert key in self.starts, ('Unable to stop tracking %s, never '
            'started tracking it' % key)

        delta = time.time() - self.starts.pop(key)
        self.data[key] += delta
        return delta

    def submit(self, *args):
        client = self.get_client(*args)
        for k in self.data.keys():
            client.send(k, self.data.pop(k))

        if settings.DEBUG:
            assert not self.starts, ('Timer(s) %r were started but never '
                'stopped' % self.starts)

    def __call__(self, key):
        return WithTimer(self, key)


class StatsdMiddleware(object):

    scope = threading.local()

    def __init__(self):
        self.scope.timings = None
        self.scope.counter = None

    @classmethod
    def start(cls, prefix='view'):
        cls.scope.timings = Timer(prefix)
        cls.scope.timings.start('total')
        cls.scope.counter = Counter(prefix)
        return cls.scope

    def process_request(self, request):
        # store the timings in the request so it can be used everywhere
        request.statsd = self.start()
        self.view_name = None

    def process_view(
            self,
            request,
            view_func,
            view_args,
            view_kwargs,
        ):

        # View name is defined as module.view
        # (e.g. django.contrib.auth.views.login)
        self.view_name = view_func.__module__ + '.' + view_func.__name__

    @classmethod
    def stop(cls, *key):
        if getattr(cls.scope, 'timings', None):
            cls.scope.timings.stop('total')
            cls.scope.timings.submit(*key)
            cls.scope.counter.submit(*key)

    def process_response(self, request, response):
        method = request.method.lower()
        if request.is_ajax():
            method += '_ajax'

        if getattr(self, 'view_name', None):
            self.stop(
                method,
                self.view_name,
            )

        self.cleanup(request)

        return response

    def process_exception(self, request, exception):
        self.cleanup(request)

    def cleanup(self, request):
        self.scope.timings = None
        self.scope.counter = None
        self.view_name = None
        request.statsd = None


class TimingMiddleware(StatsdMiddleware):
    @classmethod
    def deprecated(cls, *args, **kwargs):
        warnings.warn(
            'The `TimingMiddleware` has been deprecated in favour of '
            'the `StatsdMiddleware`. Please update your middleware settings',
            DeprecationWarning
        )

    __init__ = deprecated


class DummyWith(object):
    def __enter__(self):
        pass

    def __exit__(self, type_, value, traceback):
        pass


def start(key):
    if getattr(StatsdMiddleware.scope, 'timings', None):
        StatsdMiddleware.scope.timings.start(key)

def stop(key):
    if getattr(StatsdMiddleware.scope, 'timings', None):
        return StatsdMiddleware.scope.timings.stop(key)

def with_(key):
    if getattr(StatsdMiddleware.scope, 'timings', None):
        return StatsdMiddleware.scope.timings(key)
    else:
        return DummyWith()

def incr(key, value=1):
    if getattr(StatsdMiddleware.scope, 'counter', None):
        StatsdMiddleware.scope.counter.incr(key, value)

def decr(key, value=1):
    if getattr(StatsdMiddleware.scope, 'counter', None):
        StatsdMiddleware.scope.counter.decr(key, value)

def wrapper(prefix, f):
    @functools.wraps(f)
    def _wrapper(*args, **kwargs):
        with with_('%s.%s' % (prefix, f.__name__.lower())):
            return f(*args, **kwargs)
    return _wrapper

def decorator(prefix):
    return lambda f: wrapper(prefix, f)




from __future__ import with_statement
import django_statsd

class TimingCursorWrapper(object):
    def execute(self, *args, **kwargs):
        with django_statsd.with_('sql.%s' % self.db.alias):
            return self.cursor.execute(*args, **kwargs)

    def executemany(self, *args, **kwargs):
        with django_statsd.with_('sql.%s' % self.db.alias):
            return self.cursor.executemany(*args, **kwargs)




from __future__ import absolute_import
import django_statsd

try:
    import redis

    class StatsdRedis(redis.Redis):
        def execute_command(self, func_name, *args, **kwargs):
            with django_statsd.with_('redis.%s' % func_name.lower()):
                return origRedis.execute_command(self, func_name, *args,
                    **kwargs)

    origRedis = None
    # NOTE issubclass is true if both are the same class
    if not issubclass(redis.Redis, StatsdRedis):
        origRedis = redis.Redis
        redis.Redis = StatsdRedis
except ImportError:
    pass




import os
from setuptools import setup

if os.path.isfile('README.rst'):
    long_description = open('README.rst').read()
else:
    long_description = 'See http://pypi.python.org/pypi/django-statsd/'

version = '1.7.1'
setup(
    name='django-statsd',
    version=version,
    author='Rick van Hattem',
    author_email='Rick.van.Hattem@Fawo.nl',
    description='''django-statsd is a django app that submits query and 
        view durations to Etsy's statsd.''',
    url='https://github.com/WoLpH/django-statsd',
    license='BSD',
    packages=['django_statsd'],
    long_description=long_description,
    test_suite='nose.collector',
    setup_requires=['nose'],
    tests_require=['django'],
    install_requires=['python-statsd>=1.2'],
    classifiers=[
        'License :: OSI Approved :: BSD License',
    ],
)
